from quandoo.QuandooModel import QuandooModel


class Customer(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["id"]
        self.firstName = data["firstName"]
        self.lastName = data["lastName"]
        self.email = data["email"]
        self.phoneNumber = data["phoneNumber"]

        super().__init__(data)

    def to_json(self):
        return {
            "firstName": self.firstName,
            "lastName": self.lastName,
            "emailAddress": self.email,
            "phoneNumber": self.phoneNumber,
            "locale": "en_AU",
            "country": "AU"
        }



"""
Unofficial library, still in progress

Author(s):
    Fraser Langton <fraserbasil@gmail.com>

GitHub:
    github.com/fraser-langton/Quandoo

Quandoo API docs:
    docs.quandoo.com (/swagger.html for better docs)
"""

import json

import requests

from quandoo import config
from quandoo.Customer import Customer
from quandoo.Error import PoorResponse
from quandoo.Merchant import Merchant
from quandoo.QuandooModel import urljoin, PrettyClass
from quandoo.Reservation import Reservation


class Agent(PrettyClass):
    headers = {
        "accept": "application/json",
        "X-Quandoo-AuthToken": None
    }

    def __init__(self, oauth_token, agent_id, test=False):
        self.oauth_token = oauth_token
        self.agent_id = agent_id

        if test:
            self.url = urljoin(config.base_url_test, config.version)
        else:
            self.url = urljoin(config.base_url, config.version)

        self.headers["X-Quandoo-AuthToken"] = oauth_token

    def get_merchant(self, merchant_id):
        request = urljoin(self.url, "merchants", merchant_id)
        response = requests.get(request, headers=self.headers)

        if response.status_code == 200:
            return Merchant(json.loads(response.text), self)

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_customer(self, customer_id):
        request = urljoin(self.url, "customers", customer_id)
        response = requests.get(request, headers=self.headers)

        if response.status_code == 200:
            return Customer(json.loads(response.text), self)

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_reservation(self, reservation_id):
        request = urljoin(self.url, "reservations", reservation_id)
        response = requests.get(request, headers=self.headers)

        if response.status_code == 200:
            return Reservation(json.loads(response.text), self)

        raise PoorResponse(response.status_code, json.loads(response.text), request)



import json
import sys
import time
from datetime import datetime as dt, timezone, timedelta


class PrettyClass:
    useless_attrs = ["api_response", "agent"]

    def __str__(self):
        useful_attrs = ["{}: {}".format(key, val) for key, val in self.__dict__.items() if
                        key not in self.useless_attrs]

        return "{}(\n\t{}\n)".format(
            self.__class__.__name__,
            ",\n\t".join(useful_attrs)
        )

    def __repr__(self):
        return "\n" + indent(str(self))

    def to_tuple(self):
        return tuple([val for key, val in self.__dict__.items() if key not in self.useless_attrs])


class QuandooModel(PrettyClass):

    def __init__(self, data):
        self.api_response = data

    def get_api_response(self):
        return json.dumps(self.api_response, indent=2)


class QuandooDatetime(PrettyClass):
    TIME_RESOLUTION = 15  # minutes

    def __init__(self, year, month=None, day=None, hour=0, minute=0, second=0, microsecond=0, tzinfo=None):
        second = microsecond = 0
        if not tzinfo:
            tzinfo = timezone(timedelta(seconds=-time.timezone))

        self.datetime = dt(year, month, day, hour, minute, second, microsecond, tzinfo)
        self.__resolve_time()

    def __str__(self):
        useful_attrs = ["{}: {}".format(key, val) for key, val in self.__dict__.items() if
                        key not in self.useless_attrs] + ["{}: {}".format("q_datetime", self.get_qdt()),
                                                          "{}: {}".format("pretty_date", self.pretty_date())]

        return "{}(\n\t{}\n)".format(
            self.__class__.__name__,
            ",\n\t".join(useful_attrs)
        )

    def __eq__(self, other):
        return self.datetime == other.datetime

    def __lt__(self, other):
        return self.datetime > other.datetime

    @staticmethod
    def now():
        y, m, d, h, i, *_ = dt.now().timetuple()
        return QuandooDatetime(y, m, d, h, i)

    @staticmethod
    def parse_str_qdt(string):
        d = string.split("+")
        d[1] = d[1].replace(":", "")
        y, m, d, h, i, *_ = dt.strptime("+".join(d), "%Y-%m-%dT%H:%M:%S%z").timetuple()
        return QuandooDatetime(y, m, d, h, i)

    def __resolve_time(self):
        y, m, d, h, i, *_ = self.datetime.timetuple()
        i = ((i // QuandooDatetime.TIME_RESOLUTION) * QuandooDatetime.TIME_RESOLUTION)

        self.datetime = dt(y, m, d, h, i, tzinfo=self.datetime.tzinfo)

    def get_qdt(self):
        return dt.strftime(self.datetime, "%Y-%m-%dT%H:%M:%S{}".format(str(self.datetime.tzinfo)[-6:]))

    def pretty_date(self):
        if sys.platform.startswith("win"):
            token = "#"
        elif sys.platform.startswith("linux"):
            token = "-"
        else:
            token = ""
        return self.datetime.strftime("%{token}I:%M %p, %a %{token}d %B %Y".format(**{"token": token}))


def urljoin(*argv):
    return "/".join([str(arg) for arg in argv])


def indent(string, indent_amount=1):
    return "\n".join(["\t" * indent_amount + line for line in string.split("\n")])



class QuandooException(Exception):

    def __init__(self, error_type, error_message):
        self.error_type = error_type
        self.error_message = error_message

    def __str__(self):
        return "{}: {}".format(self.error_type, self.error_message)


class APIException(QuandooException):

    def __init__(self, status_code, response, request):
        super().__init__("{} {}".format(status_code, response["errorType"]), response["errorMessage"])
        self.request = request


class PoorResponse(APIException):

    def __init__(self, status_code, response, request):
        super().__init__(status_code, response, request)


class PoorRequest(APIException):

    def __init__(self, status_code, response, request):
        super().__init__(status_code, response, request)


class PoorType(QuandooException):

    def __init__(self, status_code, data):
        super().__init__(status_code, data)



from quandoo.Error import PoorResponse
from quandoo.QuandooModel import urljoin, QuandooModel
import json
import requests


class Reservation(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["id"]
        self.status = data["status"]
        self.startTime = data["startTime"]
        self.endTime = data["endTime"]
        self.capacity = data["capacity"]
        self.merchantId = data["merchantId"]
        self.customerId = data["customerId"]

        self.agent = agent

        super().__init__(data)

    def cancel(self):
        data = {
            "reservation": {
                "status": "CUSTOMER_CANCELED"
            }
        }

        request = urljoin(self.agent.url, "reservations", self.id)
        response = requests.patch(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            self.status = "CUSTOMER_CANCELED"
            return

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def reconfirm(self):
        data = {
            "reservation": {
                "status": "RECONFIRMED"
            }
        }

        request = urljoin(self.agent.url, "reservations", self.id)
        response = requests.patch(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            self.status = "RECONFIRMED"
            return

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def change_capacity(self, new_capacity):
        data = {
            "reservation": {
                "capacity": new_capacity,
            }
        }

        request = urljoin(self.agent.url, "reservations", self.id)
        response = requests.patch(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            self.capacity = new_capacity
            return

        raise PoorResponse(response.status_code, json.loads(response.text), request)


class NewReservation(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["reservation"]["id"]
        self.number = data["reservation"]["number"]
        self.status = data["reservation"]["status"]
        self.customerId = data["customer"]["id"]

        self.agent = agent

        super().__init__(data)

    def get_reservation(self):
        return self.agent.get_reservation(self.id)




import json
from datetime import datetime, timedelta

import requests

from quandoo.Customer import Customer
from quandoo.Error import PoorResponse
from quandoo.QuandooModel import QuandooModel, QuandooDatetime, urljoin
from quandoo.Reservation import Reservation, NewReservation


class Merchant(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["id"]
        self.name = data["name"]
        address_vals = [i if i in data["location"]["address"].keys() else "" for i in ['number', 'street', 'city', 'country']]
        address_vals[1] = " ".join(address_vals[:1])
        address_vals = address_vals[1:]
        self.address = ", ".join(address_vals)

        self.agent = agent

        super().__init__(data)

    def get_customers(self):
        request = urljoin(self.agent.url, "merchants", self.id, "customers")
        response = requests.get(request, headers=self.agent.headers)

        if response.status_code == 200:
            return [Customer(i, self.agent) for i in json.loads(response.text)["result"]]

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_reservations(self):
        request = "{}/merchants/{}/reservations".format(self.agent.url, self.id)
        response = requests.get(request, headers=self.agent.headers)

        if response.status_code == 200:
            return [Reservation(i, self.agent) for i in json.loads(response.text)["reservations"]]

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_available_times(self, pax: int, qdt: QuandooDatetime, duration=4):
        from_time, to_time = qdt.datetime, qdt.datetime + timedelta(hours=duration)
        from_hour, from_min = tuple(datetime.strftime(from_time, "%H %M").split())
        to_hour, to_min = tuple(datetime.strftime(to_time, "%H %M").split())

        payload = "times?agentId={}&capacity={}&fromTime={}%3A{}&toTime={}%3A{}".format(
            self.agent.agent_id, pax, from_hour, from_min, to_hour, to_min)

        request = urljoin(self.agent.url, "merchants", self.id, "availabilities", qdt.datetime.strftime("%Y-%m-%d"), payload)
        response = requests.get(request, headers=self.agent.headers)

        if response.status_code == 200:
            return [QuandooDatetime.parse_str_qdt(i["dateTime"]) for i in json.loads(response.text)["timeSlots"]]

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def is_available(self, pax: int, qdt: QuandooDatetime):
        return qdt in self.get_available_times(pax, qdt)

    def create_reservation(self, customer, pax: int, qdt: QuandooDatetime):
        data = {
            "reservation": {
                "merchantId": self.id,
                "capacity": pax,
                "dateTime": qdt.get_qdt()
            },
            "customer": customer.to_json(),
            "tracking": {
                "agent": {
                    "id": self.agent.agent_id
                }
            }
        }

        request = urljoin(self.agent.url, "reservations")
        response = requests.put(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            return NewReservation(json.loads(response.text), self.agent)

        raise PoorResponse(response.status_code, json.loads(response.text), request)




"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='Quandoo',
    version='1.2.7',
    description="A SDK for interacting with the Quandoo API, it is a work in progress",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/fraser-langton/Quandoo',
    author='Fraser Langton',
    author_email='fraserbasil@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='quandoo api',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4',
    install_requires=['requests'],
)
